class FaspNode

  # Get nodes in failover group other than this node
  def get_other_nodes_in_failover_group
    other_nodes = FaspNode.find(:all, :conditions => ["node_address <> ? AND node_group_name = ? AND enable_node_group IS TRUE", node_address, node_group_name]) if node_group_name.present?
    other_nodes.nil? ? [] : other_nodes
  end

  def get_all_nodes_in_failover_group
    get_other_nodes_in_failover_group << self
  end

  # Copy endpoints and their email notifications between self and other members in a failover cluster
  # 'self' has lower precedence than existing members of the group, we import from other members first
  def homogenize_cluster_endpoints!
    raise "Node must be a member of a failover group" unless node_group_name.present?

    other_member_nodes = get_other_nodes_in_failover_group

    if !other_member_nodes.empty?
      # Import from already homogenized node
      homogenize_node!(self, other_member_nodes.first)

      # Important
      reload

      # Propagate any new endpoints from this node to other nodes
      other_member_nodes.each do |other_node|
        homogenize_node!(other_node, self)
      end

      # Homogenize group transfer paths
      Group.all.each do |g|
        # Make group paths match group
        g.homogenize_node_group_paths(self, other_member_nodes.first)

        # Add any new group paths to rest of group
        other_member_nodes.each do |other_node|
          g.homogenize_node_group_paths(other_node, self)
        end
      end
    end
  end

  # Synchronize target node to already homogenized node
  def homogenize_node!(target_node, homogenized_node)
    homogenized_node.endpoints.each do |ep|
      homogenize_endpoint_on_node!(ep, target_node)
    end
  end

  # Homogenize a single endpoint between all nodes
  def homogenize_endpoint!(endpoint)
    get_other_nodes_in_failover_group.each do |node|
      homogenize_endpoint_on_node!(endpoint, node)
    end
  end

  # Homogenize a single endpoint with a single node
  def homogenize_endpoint_on_node!(source_endpoint, target_node)
    target_endpoint = target_node.endpoints.find_by_login(source_endpoint.login)

    if target_endpoint
      target_endpoint.name = target_endpoint.login == '*' ? target_endpoint.name : source_endpoint.name
      target_endpoint.port = source_endpoint.port
      target_endpoint.ssh_key_id = source_endpoint.ssh_key_id
      target_endpoint.password = source_endpoint.unencrypted_password
      target_endpoint.clear_password if source_endpoint.unencrypted_password.blank?
    else
      target_endpoint = target_node.endpoints.build(
          {
              :login => source_endpoint.login,
              :address => target_node.node_address,
              :name => source_endpoint.login == '*' ? nil : source_endpoint.name,
              :port => source_endpoint.port,
              :ssh_key_id => source_endpoint.ssh_key_id,
              :password => source_endpoint.unencrypted_password,
              :endpoint_type => source_endpoint.endpoint_type,
          }
      )
    end

    target_endpoint.save!

    # iterate though the endpoint's email notifications
    source_endpoint.email_notification_list_rows.each do |source_login_email|
      target_login_email = target_endpoint.email_notification_list_rows.find_by_email(source_login_email.email)
      if target_login_email
        target_login_email.start_template_id = source_login_email.start_template_id
        target_login_email.success_template_id = source_login_email.success_template_id
        target_login_email.error_template_id = source_login_email.error_template_id
      else
        target_login_email = target_endpoint.email_notification_list_rows.build(
            {
                :email => source_login_email.email,
                :start_template_id => source_login_email.start_template_id,
                :success_template_id => source_login_email.success_template_id,
                :error_template_id => source_login_email.error_template_id
            }
        )
      end
      target_login_email.save!
    end
  end

  # Find all other endpoints with matching logins in group
  def get_other_endpoints_in_group(login)
    other_member_nodes = get_other_nodes_in_failover_group

    return if other_member_nodes.nil?

    other_endpoints = []

    other_member_nodes.each do |other_node|
      other_endpoint = other_node.endpoints.find_by_login(login)

      other_endpoints << other_endpoint unless other_endpoint.nil?
    end

    other_endpoints
  end

  # Delete endpoints from all nodes in group
  def homogenize_delete_endpoint!(deleted_endpoint_login)
    other_endpoints = get_other_endpoints_in_group(deleted_endpoint_login)

    other_endpoints.each do |ep|
      ep.destroy unless ep.nil?
    end
  end

  # Delete endpoint email from all nodes in group
  def homogenize_delete_endpoint_email!(endpoint_login, deleted_endpoint_email)
    other_member_nodes = get_other_nodes_in_failover_group

    other_member_nodes.each do |other_node|
      other_endpoint = other_node.endpoints.find_by_login(endpoint_login)

      to_delete = other_endpoint.email_notification_list_rows.find_by_email(deleted_endpoint_email) unless other_endpoint.nil?

      to_delete.delete unless to_delete.nil?
    end
  end

end
