class NodeConfiguration::Section::Vlink < NodeConfiguration::Section::Base

  setting :trunk_id, :integer, "Vlink ID", :as_type => :positive_integer, :range => 1..255
  setting :trunk_name, :string, "Vlink Name", :length => {:maximum => 30, :allow_blank => true}
  setting :trunk_on, :string, "On", :radio => ["true", "false"]
  setting :trunk_capacity, :string, "Capacity"
  
  validates_presence_of :trunk_id, :message => "Vlink ID can't be blank"
  
  attr_accessor :visible, :default_capacity, :scheduled_rates
  
  def initialize(node, trunk_id)
    @owner = "trunk:#{trunk_id.to_i}"
    super(node)
    set_default_and_scheduled_rates
    self.trunk_on = ["true", "false"].include?(trunk_on) ? trunk_on : "true"
  end

  # Parse out default and scheduled rates (capacity):
  # * rates are delimited by ';' like: <rate-1>;...;<rate-n>;<default rate>
  # * default rate is in string like:  capacity=<rate>
  def set_default_and_scheduled_rates
    rates = (trunk_capacity || '').split(';')
    cap_as_schedule = rates.pop || ""
    self.default_capacity = cap_as_schedule.split("=").last.to_i
    self.scheduled_rates = rates.map { |sr| NodeConfiguration::Section::VlinkRate.new_from_string(self, sr) } 
  end
  
  # Returns RateCalendarWeek instance which is used for building graphical
  # representation of a Vlinks weekly schedule.
  def rate_calendar_week
    @rate_calendar_week ||= begin
      rcw = RateCalendarWeek.new(default_capacity)
      scheduled_rates.each do |sr|
        rcw.add_cron sr
      end
      rcw
    end
  end
  
  def owner
    @owner
  end

  def visible?
    true
  end
  
  def not_in_use?
    in_use_by.empty?
  end
  
  def in_use_by
    conditions = ['name like "transfer_%_bandwidth_aggregate_trunk_id" and override_value = ?', trunk_id]
    @in_use_by = node.config_settings.all(:conditions => conditions).uniq
  end
  
  def update_attributes(attributes)
    if super
      write_settings_to_node
    end
  end
  
  def save
    if super
      write_settings_to_node
    end
  end

  def id_unique
    if node.get_vlink_list.include?(trunk_id.to_s)
      errors.add(:trunk_id, "is not unique")
      false
    else
      true
    end
  end

  def is_number?(object)
    true if Float(object) rescue false
  end

  def validate
    unless is_number?(default_capacity) && (default_capacity.to_i >= 0)
      errors.add(:default_capacity, "must be a positive integer")
      set_trunk_capacity
      return
    end
    unless default_capacity.to_s =~ /^\d{1,10}$/
      errors.add(:default_capacity, "must be less than 10,000,000,000")
    end
    set_trunk_capacity
  end

  # Capacity is delimited string of rates with the default rate at the end.
  def set_trunk_capacity
    self.trunk_capacity = [scheduled_rates, default_capacity_as_schedule].flatten.map(&:to_s).join(';')
  end

  def default_capacity_as_schedule
    "#{default_capacity}"
  end
  
  # Write config to node for this vlink.
  def write_settings_to_node
    node.asconfigurator(set_config_command)
  end
  
  def set_command
    ["set_trunk_data",Console.uac_version, "id,#{trunk_id}"]
  end

  def delete
    if not_in_use?
      cmd = ["delete_trunk", Console.uac_version, %(id, #{trunk_id})].join("\n")
      node.asconfigurator(cmd)
      true
    else
      errors.add_to_base('Cannot delete this Vlink. It is in use by: ' + in_use_by.map(&:owner).uniq.join(", "))
      false
    end
  end
    
  def <=>(other)
    sort_field <=> other.sort_field
  end
  
  def sort_field
    trunk_id.to_i
  rescue
    0
  end
  
  def id_and_name
    r = trunk_id.to_s
    r += " - #{trunk_name}" unless trunk_name.blank?
    r
  end
  
  def display_name
    @display_name ||= begin
      r = id_and_name
      r += scheduled_rates.empty? ? " (#{default_capacity_display})" : ' (scheduled)'
      r
    end
  end
  
  def default_capacity_display
    "#{default_capacity}Kbps"
  end
  
  def capacity_display
    scheduled_rates_display = scheduled_rates.map(&:display)
    default_rate_display = "Default #{default_capacity_display}"
    scheduled_rates_display.push(default_rate_display).join("<br />")
  end
end